<?php
// $Id: pnuser.php,v 1.23 2004/04/19 10:37:36 markwest Exp $
// ----------------------------------------------------------------------
// Advanced Polls Module for the POST-NUKE Content Management System
// Copyright (C) 2002-2004 by Mark West
// http://www.markwest.me.uk/
// ----------------------------------------------------------------------
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
// ----------------------------------------------------------------------
// Original Author of file: Mark West
// Purpose of file: Advanced Polls User Frontend
// ----------------------------------------------------------------------

/**
* Advanced Polls Module User Frontend
* @package Advanced_Polls
* @package $Id: pnuser.php,v 1.23 2004/04/19 10:37:36 markwest Exp $
* @author Mark West <mark@markwest.me.uk> 
* @link http://www.markwest.me.uk Advanced Polls Support Site
* @copyright (C) 2002-2004 by Mark West
*/
 
/**
* the main user function
* This function is the default function, and is called whenever the module is
* initiated without defining arguments.  As such it can be used for a number
* of things, but most commonly it either just shows the module menu and
* returns or calls whatever the module designer feels should be the default
* function (often this is the view() function)
* @returns HTML output
* @author Mark West <mark@markwest.me.uk>
* @copyright (C) 2002-2004 by Mark West
* @since 1.0
* @version 1.1
*/
function advanced_polls_user_main() 
{
    // Create output object - this object will store all of our output so that
    // we can return it easily when required
	$pnRender =& new pnRender('advanced_polls');

	// Security check - important to do this as early as possible to avoid
	// potential security holes or just too much wasted processing.  For the
	// main function we want to check that the user has at least overview
	// privilege for some item within this component, or else they won't be
	// able to see anything and so we refuse access altogether.  The lowest
	// level of access for administration depends on the particular module, but
	// it is generally either 'overview' or 'read'
	if (!pnSecAuthAction(0, 'advanced_polls::', '::', ACCESS_OVERVIEW)) {
		return pnVarPrepHTMLDisplay(_ADVANCEDPOLLSNOAUTH);
	}

	// Add menu to output - it helps if all of the module pages have a standard
	// menu at their head to aid in navigation
	$pnRender->assign('main', advanced_polls_user_view());

	// Return the output that has been generated by this function
	return $pnRender->fetch('advancedpollsusermain.htm');
}

/**
* view items
* This is a standard function to provide an overview of all of the items
* available from the module.
* @returns HTML output
* @author Mark West <mark@markwest.me.uk>
* @copyright (C) 2002-2004 by Mark West
* @since 1.0
* @version 1.1
*/
function advanced_polls_user_view() 
{
	// Get parameters from whatever input we need.  All arguments to this
	// function should be obtained from pnVarCleanFromInput(), getting them
	// from other places such as the environment is not allowed, as that makes
	// assumptions that will not hold in future versions of PostNuke
	$startnum = pnVarCleanFromInput('startnum');

    // Create output object - this object will store all of our output so that
    // we can return it easily when required
	$pnRender =& new pnRender('advanced_polls');

	// Security check - important to do this as early as possible to avoid
	// potential security holes or just too much wasted processing
	if (!pnSecAuthAction(0, 'advanced_polls::', '::', ACCESS_OVERVIEW)) {
		return pnVarPrepHTMLDisplay(_ADVANCEDPOLLSNOAUTH);
	}

	// Load API.  All of the actual work for obtaining information on the items
	// is done within the API, so we need to load that in before we can do
	// anything.  If the API fails to load an appropriate error message is
	// posted and the function returns
	if (!pnModAPILoad('advanced_polls', 'user')) {
		return pnVarPrepHTMLDisplay(_LOADFAILED);
	}

	// The API function is called.  The arguments to the function are passed in
	// as their own arguments array
	$items = pnModAPIFunc('advanced_polls',	'user',	'getall', array('startnum' => $startnum));

	// The return value of the function is checked here, and if the function
	// suceeded then an appropriate message is posted.  Note that if the
	// function did not succeed then the API function should have already
	// posted a failure message so no action is required
	if ($items == false) {
		return pnVarPrepHTMLDisplay(_ADVANCEDPOLLSITEMFAILED);
	}

	//----------------------------------------------------------------------------
	// display polls that are currently active
	//----------------------------------------------------------------------------
    $activepolls = array();
	foreach ($items as $item) {

		$fullitem = pnModAPIFunc('advanced_polls', 'user', 'get', array('pollid' => $item['pollid']));

		// is this poll currently open for voting
		$ispollopen = pnModAPIFunc('advanced_polls', 'user', 'isopen', array('pollid' => $fullitem['pn_pollid']));

		// is this user/ip etc. allowed to vote under voting regulations
		$isvoteallowed = pnModAPIFunc('advanced_polls',	'user', 'isvoteallowed', array('pollid' => $fullitem['pn_pollid']));

		if ($fullitem['pn_opendate'] > time()) {
			$notyetopen = true;
		} else {
			$notyetopen = false;
		}

		if (pnSecAuthAction(0, 'advanced_polls::item', "$item[polltitle]::$item[pollid]", ACCESS_READ)) {

			if ($ispollopen == true) {
				//and ($isvoteallowed == true))
									 
				$options = array();
				if (pnSecAuthAction(0, 'advanced_polls::item', "$item[polltitle]::$item[pollid]", ACCESS_COMMENT)) {
					if ($isvoteallowed == true) {
						$options[] = array('url' => pnModURL('advanced_polls', 'user', 'display', array('pollid' => $item['pollid'])),
										   'title' => _ADVANCEDPOLLSVOTE);
					} else {
						$options[] = array('url' => pnModURL('advanced_polls', 'user', 'display', array('pollid' => $item['pollid'], 'results' => 1)),
							               'title' => _ADVANCEDPOLLSRESULTS);
					}
				}
				if (pnSecAuthAction(0, 'advanced_polls::item', "$item[polltitle]::$item[pollid]", ACCESS_EDIT)) {
					$options[] = array('url' => pnModURL('advanced_polls', 'admin', 'modify', array('pollid' => $item['pollid'])),
									   'title' => _EDIT);
				}

				if ($fullitem['pn_closedate'] == 0) {
					$closedate = _ADVANCEDPOLLSNOCLOSEDATE;
				} else {
					$closedate = ml_ftime(constant(pnModGetVar('advanced_polls', 'userdateformat')), $fullitem['pn_closedate']);
				}
				$activepolls[]=  array('url' => pnModURL('advanced_polls', 'user', 'display', array('pollid' => $item['pollid'])),
					                   'title' => $item['polltitle'],
									   'closedate' => $closedate,
									   'options' => $options);
			}
		}
	}
    $pnRender->assign('activepolls', $activepolls);
	//----------------------------------------------------------------------------
	// Output Polls that have not opened yet
	//----------------------------------------------------------------------------
    $futurepolls = array();
	foreach ($items as $item) {

		$fullitem = pnModAPIFunc('advanced_polls', 'user', 'get', array('pollid' => $item['pollid']));

		// is this poll currently open for voting
		$ispollopen = pnModAPIFunc('advanced_polls', 'user', 'isopen', array('pollid' => $item['pollid']));

		// is this user/ip etc. allowed to vote under voting regulations
		$isvoteallowed = pnModAPIFunc('advanced_polls',	'user',	'isvoteallowed', array('pollid' => $item['pollid']));

		if ($fullitem['pn_opendate'] > time()) {
			$notyetopen = true;
		} else {
			$notyetopen = false;
		}

		if (pnSecAuthAction(0, 'advanced_polls::item', "$item[polltitle]::$item[pollid]", ACCESS_READ)) {

			if (($ispollopen == false) and ($isvoteallowed == true) and ($notyetopen == true)) {

				$options = array();
				if (pnSecAuthAction(0, 'advanced_polls::item', "$item[polltitle]::$item[pollid]", ACCESS_COMMENT)) {
					$options[] = array('url' => pnModURL('advanced_polls', 'user', 'display', array('pollid' => $item['pollid'])),
						               'title' => _ADVANCEDPOLLSPREVIEW);
				}
				$futurepolls[] = array('url' => pnModURL('advanced_polls', 'user', 'display', array('pollid' => $item['pollid'])),
					                   'title' => $item['polltitle'],
								       'opendate' => ml_ftime(constant(pnModGetVar('advanced_polls', 'userdateformat')), $fullitem['pn_opendate']),
									   'options' => $options);
			}
		}
	}
    $pnRender->assign('futurepolls', $futurepolls);

	//----------------------------------------------------------------------------
	// Output Polls that have closed
	//----------------------------------------------------------------------------
    $closedpolls = array();
	foreach ($items as $item) {

		$fullitem = pnModAPIFunc('advanced_polls', 'user', 'get', array('pollid' => $item['pollid']));

		// is this poll currently open for voting
		$ispollopen = pnModAPIFunc('advanced_polls', 'user', 'isopen', array('pollid' => $item['pollid']));

		// is this user/ip etc. allowed to vote under voting regulations
		// show all closed polls in the previous poll list
		//$isvoteallowed = pnModAPIFunc('advanced_polls',
		//	'user',
		//	'isvoteallowed',
		//	array('pollid' => $item['pollid']));

		if ($fullitem['pn_opendate'] > time()) {
			$notyetopen = true;
		} else {
			$notyetopen = false;
		}

		// Security check - important to do this as early as possible to avoid
		// potential security holes or just too much wasted processing
		if (pnSecAuthAction(0, 'advanced_polls::item', "$item[polltitle]::$item[pollid]", ACCESS_READ)) {

			//if (($ispollopen == false) and ($isvoteallowed == true) and ($notyetopen == false)) {
			if (($ispollopen == false) and ($notyetopen == false)) {
				 
				$options = array();
				if (pnSecAuthAction(0, 'advanced_polls::item', "$item[polltitle]::$item[pollid]", ACCESS_COMMENT)) {
					$options[] = array('url' => pnModURL('advanced_polls', 'user', 'display', array('pollid' => $item['pollid'])),
						               'title' => _ADVANCEDPOLLSRESULTS);
				}
                $closedpolls[] = array('url' => pnModURL('advanced_polls', 'user', 'display', array('pollid' => $item['pollid'])),
					                   'title' => $item['polltitle'],
									   'opendate' => ml_ftime(constant(pnModGetVar('advanced_polls', 'userdateformat')), $fullitem['pn_opendate']),
									   'options' => $options);
			}
		}
	}
    $pnRender->assign('closedpolls', $closedpolls);

	// Return the output that has been generated by this function
	return $pnRender->fetch('advancedpollsuserview.htm');
}

/**
* display item
* This is a standard function to provide detailed informtion on a single poll
* available from the module.
* @param $args['pollid'] Poll id to display
* @param $args['results'] 1 to show results components 0 otherwise
* @returns HTML output
* @author Mark West <mark@markwest.me.uk>
* @copyright (C) 2002-2004 by Mark West
* @since 1.0
* @version 1.1
*/
function advanced_polls_user_display($args) 
{
	// Get parameters from whatever input we need.  All arguments to this
	// function should be obtained from pnVarCleanFromInput(), getting them
	// from other places such as the environment is not allowed, as that makes
	// assumptions that will not hold in future versions of PostNuke
	$pollid = pnVarCleanFromInput('pollid');
	$results = pnVarCleanFromInput('results');
	
	// User functions of this type can be called by other modules.  If this happens
	// then the calling module will be able to pass in arguments to this function
	// through the $args parameter.  Hence we extract these arguments *after* we have
	// obtained any form-based input through pnVarCleanFromInput().
	extract($args);

    // Create output object - this object will store all of our output so that
    // we can return it easily when required
	$pnRender =& new pnRender('advanced_polls');

	// get theme name
	$pnRender->assign('theme', pnUserGetTheme());

	// Load API.  All of the actual work for obtaining information on the items
	// is done within the API, so we need to load that in before we can do
	// anything.  If the API fails to load an appropriate error message is
	// posted and the function returns
	if (!pnModAPILoad('advanced_polls', 'user')) {
		return pnVarPrepHTMLDisplay(_LOADFAILED);
	}

	// The API function is called.  The arguments to the function are passed in
	// as their own arguments array
	$item = pnModAPIFunc('advanced_polls', 'user', 'get', array('pollid' => $pollid));

	$polloptionarray = array();
	$polloptionarray = $item['pn_optionarray'];

	// The return value of the function is checked here, and if the function
	// suceeded then an appropriate message is posted.  Note that if the
	// function did not succeed then the API function should have already
	// posted a failure message so no action is required
	if ($item == false) {
		return pnVarPrepHTMLDisplay(_ADVANCEDPOLLSITEMFAILED);
	}

	// check if we need to reset any poll votes
	$resetrecurring = pnModAPIFunc('advanced_polls', 'user', 'resetrecurring', array('pollid' => $pollid));

	// Security check - important to do this as early as possible to avoid
	// potential security holes or just too much wasted processing
	if (pnSecAuthAction(0, 'advanced_polls::item', "$item[pn_title]::$item[pn_pollid]", ACCESS_READ)) {

		// is this poll currently open for voting
		$ispollopen = pnModAPIFunc('advanced_polls', 'user', 'isopen', array('pollid' => $item['pn_pollid']));
	
		// is this user/ip etc. allowed to vote under voting regulations
		$isvoteallowed = pnModAPIFunc('advanced_polls', 'user', 'isvoteallowed', array('pollid' => $item['pn_pollid']));

		// get vote counts
		$votecount = pnModAPIFunc('advanced_polls', 'user', 'pollvotecount', array('pollid' => $pollid));

		if ($item['pn_opendate'] > time()) {
			$notyetopen = true;
		} else {
			$notyetopen = false;
		}

		$pnRender->assign('pollid', $pollid);
		$pnRender->assign('item', $item);

		// Now lets work out which view to display
		if ($results) { $displayresults = true; }
		if ($ispollopen && $isvoteallowed) { $displayvotingform = true;}
		if (!$isvoteallowed || (!$ispollopen && !$notyetopen)) { $displayresults = true; }
		if ($notyetopen) { $displaypreview = true; }
		

		//----------------------------------------------------------------------------
		// if poll is open, voting is allowed then display voting form
		//----------------------------------------------------------------------------
		if ($displayvotingform) {

			$pnRender->assign('polltype', $item['pn_multipleselect']);
			$pnRender->assign('multiplecount', $item['pn_multipleselectcount']);
	
			$options = array();
			for ($i = 0, $max = count($polloptionarray); $i < $max; $i++) {
				$optiontext = $polloptionarray[$i]['optiontext'];
				$optioncolor = $polloptionarray[$i]['optioncolour'];
				$voteid = $polloptionarray[$i]['voteid'];
				if ($optiontext) {
					$options[] = array('optiontext' => $optiontext,
									   'optioncolor' => $optioncolor,
									   'voteid' => $voteid);
				}
			}
			$pnRender->assign('options', $options);
			$template = 'advancedpollsuservotingform';
		} elseif ($displayresults)  {
			//----------------------------------------------------------------------------
			// Output results graph if poll has closed/ is not open yet or
			// if results have been spefifically requested
			//----------------------------------------------------------------------------
			$pnRender->assign('ispollopen', $ispollopen);
	
			// display poll results
			$pollresults = array();
			for ($i = 0, $max = count($polloptionarray); $i < $max; $i++) {
				$optiontext = $polloptionarray[$i]['optiontext'];
				$optioncolor = $polloptionarray[$i]['optioncolour'];
				$voteid = $polloptionarray[$i]['voteid'];
				// if the poll option has some text then display its result
	
				if ($optiontext) {
					$row = array();
					// calculate vote percentage and scaled percentage for graph
	
					if ($votecount['pn_votecountarray'][$i+1]  != 0) {
						$percent = ($votecount['pn_votecountarray'][$i+1] / $votecount['pn_totalvotecount']) * 100;
					} else {
						$percent = 0;
					}
					$percentint = (int)$percent;
					$percentintscaled = $percentint * pnModGetVar('advanced_polls', 'scalingfactor');
	
					// output poll option text
					$pollresults[] = array('optiontext' => $optiontext,
									       'optioncolor' => $optioncolor,
									       'percentage' => $percentint,
									       'scaledpercentage' => $percentintscaled,
									       'votecount' => $votecount['pn_votecountarray'][$i+1]);
				}
			}
			$pnRender->assign('results', $pollresults);
			$template = 'advancedpollsuserresults';
		} elseif ($displaypreview) {
			//----------------------------------------------------------------------------
			// Output details of poll if poll is not open yet
			//----------------------------------------------------------------------------
			$votecount = pnModAPIFunc('advanced_polls', 'user', 'pollvotecount', array('pollid' => $pollid));
	
			// display poll results
			$options = array();
			for ($i = 0, $max = count($polloptionarray); $i < $max; $i++) {
				$options[] = array('optiontext' => $polloptionarray[$i]['name']);
			}
			$pnRender->assign('options', $options);
	
			// close results table out
			$template = 'advancedpollsuserfuturepoll';
		} else {
			return pnVarPrepHTMLDisplay(_ADVANCEDPOLLSNOTEMPLATESELECTED);
		}
	} else {
		return pnVarPrepHTMLDisplay(_ADVANCEDPOLLSNOAUTH);
	}

    // Let any hooks know that we are displaying an item.  As this is a display
    // hook we're passing a URL as the extra info, which is the URL that any
    // hooks will show after they have finished their own work.  It is normal
    // for that URL to bring the user back to this function
    $pnRender->assign('hooks' ,pnModCallHooks('item',
                                              'display',
                                              $pollid,
                                              pnModURL('advanced_polls',
                                                       'user',
                                                       'display',
                                                       array('pollid' => $pollid))));

	// Return the output that has been generated by this function
	if ($pnRender->template_exists($template.$pollid.'.htm')) {
		return $pnRender->fetch($template.$pollid.'.htm');
	} else {
		return $pnRender->fetch($template.'.htm');
	}

}

/**
* Process voting form
* @returns HTML output
* @author Mark West <mark@markwest.me.uk>
* @copyright (C) 2002-2004 by Mark West
* @since 1.0
* @version 1.1
*/
function advanced_polls_user_vote($args) 
{
	// get variables submitted from form
	$pollid = pnVarCleanFromInput('pollid');
	$results = pnVarCleanFromInput('results');
	$multiple = pnVarCleanFromInput('multiple');
	$multiplecount = pnVarCleanFromInput('multiplecount');
	$polltitle = pnVarCleanFromInput('polltitle');
	$optioncount =  pnVarCleanFromInput('optioncount');
	$polldisplayresults = pnVarCleanFromInput('polldisplayresults');
	$returnurl = pnVarCleanFromInput('returnurl');

	// User functions of this type can be called by other modules.  If this happens
	// then the calling module will be able to pass in arguments to this function
	// through the $args parameter.  Hence we extract these arguments *after* we have
	// obtained any form-based input through pnVarCleanFromInput().
	extract($args); 

	if (!isset($results)) {
		$results = 0;
	}
	if (!isset($multiple)) {
		$multiple = 0;
	}

	if (pnSecAuthAction(0,'advanced_polls::item',"$polltitle::$pollid",ACCESS_OVERVIEW)) {
	    if (pnSecAuthAction(0,'advanced_polls::item',"$polltitle::$pollid",ACCESS_COMMENT)) {
			if (!pnModAPILoad('advanced_polls', 'user')) {
				pnSessionSetVar('errormsg', _LOADFALED);
				return true;
			}

			// call api function to establish if poll is currently open
			$ispollopen = pnModAPIFunc('advanced_polls', 'user', 'isopen', array('pollid' => $pollid));

			// if the poll is open then start to add the current vote
			if ($ispollopen == true) {
				// is this vote allowed under voting regulations
				$isvoteallowed = pnModAPIFunc('advanced_polls', 'user', 'isvoteallowed', array('pollid' => $pollid));

				// if vote is allowed then add vote to db tables
				if ($isvoteallowed == true) {
					 if ($multiple == 1) {
						 if ($multiplecount == -1) {
							 for ($i = 1, $max = $optioncount; $i <= $max; $i++) {
								 $voteid = pnVarCleanFromInput('option' . ($i));
								 if ($voteid != null) {
									 $result = pnModAPIFunc('advanced_polls', 'user', 'addvote',
															array('pollid' => $pollid,
																  'polltitle' => $polltitle,
																  'voteid' => $i,
																  'voterank' => 1));
								 }
							 }
						} else {
							for ($i = 1, $max = $multiplecount; $i <= $max; $i++) {
								$voteid = pnVarCleanFromInput('option' . ($i));
								$result = pnModAPIFunc('advanced_polls','user','addvote',
									array('pollid' => $pollid,
										  'polltitle' => $polltitle,
										  'voteid' => $voteid,
										  'voterank' => $i));
							 }
						}
					} else {
						$voteid = pnVarCleanFromInput('option'.$pollid);
						$result = pnModAPIFunc('advanced_polls','user','addvote',
							array('pollid' => $pollid,
								  'polltitle' => $polltitle,
								  'voteid' => $voteid,
								  'voterank' => 1));
					}
				}
			}
		}
	}

	if (($polldisplayresults == 0) && isset($polldisplayresults) && isset($returnurl)) {
		pnRedirect($returnurl);
	} else {
		pnRedirect(pnModURL('advanced_polls','user','display',
							 array('pollid' => $pollid,
								   'results' => $results)));
	}
	return true;
}

?>